using System;
using System.Text;
//using org.bouncycastle.math;
//using org.bouncycastle.math.ec;
//using org.bouncycastle.security;
using org.bouncycastle.crypto;
//using org.bouncycastle.crypto.signers;
using org.bouncycastle.crypto.digests;
//using org.bouncycastle.crypto.engines;
//using org.bouncycastle.crypto.agreement;
//using org.bouncycastle.crypto.generators;
//using org.bouncycastle.crypto.parameters;
//using org.bouncycastle.crypto.macs;
//using org.bouncycastle.crypto.modes;
//using org.bouncycastle.crypto.paddings;
using org.bouncycastle.util.encoders;
using org.bouncycastle.util.test;


namespace test.crypto
{
	/**
	 * RIPEMD128 Digest Test
	 */
	public class RIPEMD128DigestTest: Test
	{
		readonly static String[] messages = 
		{
			"",
			"a",
			"abc",
			"message digest",
			"abcdefghijklmnopqrstuvwxyz",
			"abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
			"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789",
			"12345678901234567890123456789012345678901234567890123456789012345678901234567890"
		};

		readonly static String[] digests = {
			"cdf26213a150dc3ecb610f18f6b38b46",
			"86be7afa339d0fc7cfc785e72f578d33",
			"c14a12199c66e4ba84636b0f69144c77",
			"9e327b3d6e523062afc1132d7df9d1b8",
			"fd2aa607f71dc8f510714922b371834e",
			"a1aa0689d0fafa2ddc22e88b49133a06",
			"d1e959eb179c911faea4624c60c5c702",
			"3f45ef194732c2dbb2c4a2c769795fa3"
		};

		readonly static String million_a_digest = "4a7f5723f954eba1216c9d8f6320431f";

		public static bool arraysEqual(byte[] a, byte[] b)
		{
			if (a == b) return true;
			if (a.Length != b.Length) return false;

			for (int i = 0; i < a.Length; i++)
			{
				if (a[i] != b[i]) return false;
			}

			return true;
		}
        
		public String getName()
		{
			return "RIPEMD128";
		}

		public TestResult perform()
		{
			Digest digest = new RIPEMD128Digest();
			byte[] resBuf = new byte[digest.getDigestSize()];

			for (int i = 0; i < messages.Length; i++)
			{
				byte[] m = getBytes(messages[i]);
				digest.update(m, 0, m.Length);
				digest.doFinal(resBuf, 0);

				if (!arraysEqual(resBuf, Hex.decode(digests[i])))
				{
					return new SimpleTestResult(false, getName() + ": Vector " + i + " failed");
				}
			}

			//
			// test 2
			//
			byte[] mm = getBytes(messages[messages.Length-1]);

			digest.update(mm, 0, mm.Length/2);

			// clone the Digest
			Digest d = new RIPEMD128Digest((RIPEMD128Digest)digest);

			digest.update(mm, mm.Length/2, mm.Length - mm.Length/2);
			digest.doFinal(resBuf, 0);

			if (!arraysEqual(resBuf, Hex.decode(digests[digests.Length-1])))
			{
				return new SimpleTestResult(false,
					"RIPEMD128 failing clone test"
					+ System.Environment.NewLine
					+ "    expected: " + digests[digests.Length-1]
					+ System.Environment.NewLine
					+ "    got     : " + ByteArray2String(Hex.encode(resBuf)));
			}

			d.update(mm, mm.Length/2, mm.Length - mm.Length/2);
			d.doFinal(resBuf, 0);

			if (!arraysEqual(resBuf, Hex.decode(digests[digests.Length-1])))
			{
				return new SimpleTestResult(false,
					"RIPEMD128 failing clone test - part 2"
					+ System.Environment.NewLine
					+ "    expected: " +  digests[digests.Length-1]
					+ System.Environment.NewLine
					+ "    got     : " + ByteArray2String(Hex.encode(resBuf)));
			}

			for (int i = 0; i < 1000000; i++)
			{
				digest.update((byte)'a');
			}
			digest.doFinal(resBuf, 0);

			if (!arraysEqual(resBuf, Hex.decode(million_a_digest)))
			{
				return new SimpleTestResult(false, getName() + ": Million a's failed");
			}

			return new SimpleTestResult(true, getName() + ": Okay");
		}

		private byte [] getBytes(String str)
		{
			char[]  cs = str.ToCharArray();
			byte[]  bs = new byte[cs.Length];

			for (int i = 0; i != cs.Length; i++)
			{
				bs[i] = Convert.ToByte(cs[i]);
			}
			return bs;
		}
		private String ByteArray2String(byte[] bs)
		{
			StringBuilder s = new StringBuilder();
			for (int i = 0; i < bs.Length; i++) 
			{
				s.Append(Convert.ToChar(bs[i]));
			}
			return s.ToString();
		}


		[STAThread]
		static void Main(string[] args)
		{
			RIPEMD128DigestTest test = new RIPEMD128DigestTest();
			TestResult          result = test.perform();

			Console.WriteLine(result.ToString());
		}
	}

}
