using System;
using System.IO;
using System.Collections;
using org.bouncycastle.io;

namespace org.bouncycastle.asn1
{
	public class BERInputStream : DERInputStream
	{
				//private DERObject END_OF_STREAM = new DERObject();
		private class DEReos : DERObject
		{
			public override void encode(DEROutputStream outStream)
			{
				throw new IOException("Eeek!");
			}
		}
		private DERObject END_OF_STREAM = new DEReos();

//				private DERObject END_OF_STREAM = new DERObject() {
//													void encode(
//														DEROutputStream outStream)
//													//throws IOException
//													{
//														throw new IOException("Eeek!");
//													}
//		
//												};
		public BERInputStream(
			Stream ins) : base(ins) {}


		/**
		* read a string of bytes representing an indefinite length object.
		*/
		private byte[] readIndefiniteLengthFully()
			//throws IOException
		{
			MemoryStream   bOut = new MemoryStream();
			int                     b, b1;

			b1 = read();

			while ((b = read()) >= 0)
			{
				if (b1 == 0 && b == 0)
				{
					break;
				}

				bOut.WriteByte((byte)b1);
				b1 = b;
			}

			return bOut.ToArray();
		}

		private BERConstructedOctetString buildConstructedOctetString()
			//throws IOException
		{
			ArrayList                  octs = new ArrayList();

			for (;;)
			{
				DERObject		o = readObject();

				if (o == END_OF_STREAM)
				{
					break;
				}

				octs.Add(o);
			}

			return new BERConstructedOctetString(octs);
		}

		public override DERObject readObject()
			//throws IOException
		{
			int tag = read();
			if (tag == -1)
			{
				throw new EOFException();
			}
	    
			int     length = readLength();

			if (length < 0)    // indefinite length method
			{

				if (tag ==  DERTags.NULL) return null;
				else if (tag ==  (DERTags.SEQUENCE | DERTags.CONSTRUCTED))	
				{
					BERConstructedSequence  seq = new BERConstructedSequence();
	
					for (;;)
					{
						DERObject   obj = readObject();

						if (obj == END_OF_STREAM)
						{
							break;
						}

						seq.addObject(obj);
					}
					return seq;
				}
				else if (tag ==  (DERTags.OCTET_STRING | DERTags.CONSTRUCTED))
				{
					return buildConstructedOctetString();
				}
				else if (tag ==  (DERTags.SET | DERTags.CONSTRUCTED))
				{
					ASN1EncodableVector  v = new ASN1EncodableVector();
	    
					for (;;)
					{
						DERObject   obj = readObject();

						if (obj == END_OF_STREAM)
						{
							break;
						}

						v.add(obj);
					}
					return new BERSet(v);
				}
				else
				{
					//
					// with tagged object tag number is bottom 5 bits
					//
					if ((tag & DERTags.TAGGED) != 0)  
					{
						if ((tag & 0x1f) == 0x1f)
						{
							throw new IOException("unsupported high tag encountered");
						}

						//
						// simple type - implicit... return an octet string
						//
						if ((tag & DERTags.CONSTRUCTED) == 0)
						{
							byte[]  bytes = readIndefiniteLengthFully();

							return new BERTaggedObject(false, tag & 0x1f, new DEROctetString(bytes));
						}

						//
						// either constructed or explicitly tagged
						//
						DERObject		dObj = readObject();

						if (dObj == END_OF_STREAM)     // empty tag!
						{
							return new DERTaggedObject(tag & 0x1f);
						}

						DERObject       next = readObject();

						//
						// explicitly tagged (probably!) - if it isn't we'd have to
						// tell from the context
						//
						if (next == END_OF_STREAM)
						{
							return new BERTaggedObject(tag & 0x1f, dObj);
						}

						//
						// another implicit object, we'll create a sequence...
						//
						BERConstructedSequence seq = new BERConstructedSequence();

						seq.addObject(dObj);

						do
						{
							seq.addObject(next);
							next = readObject();
						}
						while (next != END_OF_STREAM);

						return new BERTaggedObject(false, tag & 0x1f, seq);
					}

					throw new IOException("unknown BER object encountered");
				}
			}
			else
			{
				if (tag == 0 && length == 0)    // end of contents marker.
				{
					return END_OF_STREAM;
				}

				byte[]  bytes = new byte[length];
	    
				readFully(bytes);
	    
				return buildObject(tag, bytes);
			}
		}

	}

}