using System;
using org.bouncycastle.math;
using org.bouncycastle.asn1;
using org.bouncycastle.asn1.x509;

namespace org.bouncycastle.asn1.pkcs
{
	public class MacData: DEREncodable
	{
		DigestInfo                  digInfo;
		byte[]                      salt;
		BigInteger                  iterationCount;

		public static MacData getInstance(
			Object  obj)
		{
			if (typeof(MacData).IsInstanceOfType(obj))
			{
				return (MacData)obj;
			}
			else if (typeof(ASN1Sequence).IsInstanceOfType(obj))
			{
				return new MacData((ASN1Sequence)obj);
			}

			throw new ArgumentException("unknown object in factory");
		}

		public MacData(
			ASN1Sequence seq)
		{
			this.digInfo = DigestInfo.getInstance(seq.getObjectAt(0));

			this.salt = ((ASN1OctetString)seq.getObjectAt(1)).getOctets();

			if (seq.size() == 3)
			{
				this.iterationCount = ((DERInteger)seq.getObjectAt(2)).getValue();
			}
			else
			{
				this.iterationCount = BigInteger.valueOf(1);
			}
		}

		public MacData(
			DigestInfo  digInfo,
			byte[]      salt,
			int         iterationCount)
		{
			this.digInfo = digInfo;
			this.salt = salt;
			this.iterationCount = BigInteger.valueOf(iterationCount);
		}

		public DigestInfo getMac()
		{
			return digInfo;
		}

		public byte[] getSalt()
		{
			return salt;
		}

		public BigInteger getIterationCount()
		{
			return iterationCount;
		}

		public DERObject getDERObject()
		{
			ASN1EncodableVector  v = new ASN1EncodableVector();

			v.add(digInfo);
			v.add(new DEROctetString(salt));
			v.add(new DERInteger(iterationCount));

			return new DERSequence(v);
		}
	}

}