using System;
using System.Collections;
using org.bouncycastle.asn1;
using org.bouncycastle.asn1.x509;

namespace org.bouncycastle.asn1.pkcs
{
	/**
	 * @deprecated - use AlgorithmIdentifier and PBES2Params
	 */
	public class PBES2Algorithms: AlgorithmIdentifier 
	{
		private DERObjectIdentifier objectId;
		private KeyDerivationFunc   func;
		private EncryptionScheme    scheme;

		public PBES2Algorithms(
			ASN1Sequence  obj): base(obj)
		{
			IEnumerator     e = obj.getObjects();
			e.MoveNext();
			objectId = (DERObjectIdentifier)e.Current;
			e.MoveNext();
			ASN1Sequence seq = (ASN1Sequence)e.Current;

			e = seq.getObjects();

			e.MoveNext();
			ASN1Sequence  funcSeq = (ASN1Sequence)e.Current;

			if (funcSeq.getObjectAt(0).Equals(PKCSObjectIdentifiers.id_PBKDF2))
			{
				func = new PBKDF2Params(funcSeq);
			}
			else
			{
				func = new KeyDerivationFunc(funcSeq);
			}

			e.MoveNext();
			scheme = new EncryptionScheme((ASN1Sequence)e.Current);
		}

		public override DERObjectIdentifier getObjectId()
		{
			return objectId;
		}

		public KeyDerivationFunc getKeyDerivationFunc()
		{
			return func;
		}

		public EncryptionScheme getEncryptionScheme()
		{
			return scheme;
		}

		public override  DERObject getDERObject()
		{
			ASN1EncodableVector  v = new ASN1EncodableVector();
			ASN1EncodableVector  subV = new ASN1EncodableVector();

			v.add(objectId);

			subV.add(func);
			subV.add(scheme);
			v.add(new DERSequence(subV));

			return new DERSequence(v);
		}
	}

}