using System;
using org.bouncycastle.math;
using org.bouncycastle.asn1;

namespace org.bouncycastle.asn1.pkcs
{
	public class PKCS12PBEParams: DEREncodable
	{
		DERInteger      iterations;
		ASN1OctetString iv;

		public PKCS12PBEParams(
			byte[]      salt,
			int         iterations)
		{
			this.iv = new DEROctetString(salt);
			this.iterations = new DERInteger(iterations);
		}

		public PKCS12PBEParams(
			ASN1Sequence  seq)
		{
			iv = (ASN1OctetString)seq.getObjectAt(0);
			iterations = (DERInteger)seq.getObjectAt(1);
		}

		public static PKCS12PBEParams getInstance(
			Object  obj)
		{
			
			if (typeof(PKCS12PBEParams).IsInstanceOfType(obj))
			{
				return (PKCS12PBEParams)obj;
			}
			else if (typeof(ASN1Sequence).IsInstanceOfType(obj))
			{
				return new PKCS12PBEParams((ASN1Sequence)obj);
			}

			throw new ArgumentException("unknown object in factory");
		}

		public BigInteger getIterations()
		{
			return iterations.getValue();
		}

		public byte[] getIV()
		{
			return iv.getOctets();
		}

		public DERObject getDERObject()
		{
			ASN1EncodableVector  v = new ASN1EncodableVector();

			v.add(iv);
			v.add(iterations);

			return new DERSequence(v);
		}
	}

}