
using System;
using System.Text;
//using org.bouncycastle.io;
using org.bouncycastle.asn1;

namespace org.bouncycastle.asn1.x509
{
	/**
	 * class for breaking up an X500 Name into it's component tokens, ala
	 * java.util.StringTokenizer. We need this class as some of the
	 * lightweight Java environment don't support classes like
	 * StringTokenizer.
	 */
	public class X509NameTokenizer
	{
		private String          oid;
		private int             index;
		private StringBuilder    buf = new StringBuilder();

		public X509NameTokenizer(
			String oid)
		{
			this.oid = oid;
			this.index = -1;
		}

		public bool hasMoreTokens()
		{
			return (index != oid.Length);
		}

		public String nextToken()
		{
			if (index == oid.Length)
			{
				return null;
			}

			int     end = index + 1;
			bool quoted = false;
			bool escaped = false;

			buf.Length = 0;

			while (end != oid.Length)
			{
				char    c = oid[end];

				if (c == '"')
				{
					if (!escaped)
					{
						quoted = !quoted;
					}
					else
					{
						buf.Append(c);
					}
					escaped = false;
				}
				else
				{
					if (escaped || quoted)
					{
						buf.Append(c);
						escaped = false;
					}
					else if (c == '\\')
					{
						escaped = true;
					}
					else if (c == ',')
					{
						break;
					}
					else
					{
						buf.Append(c);
					}
				}
				end++;
			}

			index = end;
			return buf.ToString().Trim();
		}
	}

}